package com.biganto.visual.roompark.data.data_provider

import com.biganto.visual.roompark.data.repository.api.room_park.IRoomParkApi
import com.biganto.visual.roompark.data.repository.api.room_park.raw.SubscriptionStatusRaw
import com.biganto.visual.roompark.data.repository.db.IDb
import com.biganto.visual.roompark.data.repository.db.requrey.model.SubscriptionEntity
import com.biganto.visual.roompark.data.repository.db.requrey.model.UserEntity
import com.biganto.visual.roompark.domain.contract.SubscriptionContract
import com.biganto.visual.roompark.domain.model.SubscriptionModel
import com.biganto.visual.roompark.domain.model.fromEntity
import io.reactivex.Completable
import io.reactivex.Observable
import timber.log.Timber
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 25.03.2020.
 */

private const val SUBSCRIPTION_RESULT_STATUS = "OK"

class SubscriptionRepository  @Inject constructor(
    private val api: IRoomParkApi,
    private val db: IDb
): SubscriptionContract{


    override fun saveSubscribeState(sub:SubscriptionEntity): Observable<SubscriptionEntity> {
        return saveSubscribeState(
            sub.owner as UserEntity,
            sub.id,
            sub.topic,
            sub.number,
            sub.state
            )
    }

    override fun saveSubscribeState(
        userEntity: UserEntity,
        subInnerId:Int?,
        topic: String,
        topic_id: String?,
        newState:Boolean
    ): Observable<SubscriptionEntity> {
        val userSubs = userEntity.subscriptions

        val sub = subInnerId?.let { id ->
            userSubs?.firstOrNull { sub -> sub.id == id } as SubscriptionEntity?
        }?:SubscriptionEntity()
                .apply {
                    setOwner(userEntity)
                    setTopic(topic)
                    setNumber(topic_id)
                }

        sub.setState(newState)
        return db.saveSubscription(sub)
    }


    fun saveSubscribtions(
        userEntity: UserEntity,
        apiSubs: List<SubscriptionStatusRaw>
    ): Observable<List<SubscriptionModel>>? {
        val userSubs = userEntity.subscriptions
        val newSubList = mutableListOf<SubscriptionEntity>()

        apiSubs.forEach { apiSub ->
            val cachedSub =
                userSubs?.firstOrNull { s ->
                    s.topic == apiSub.topic && s.number == apiSub.estate_id
                }
                        as SubscriptionEntity?
                    ?: SubscriptionEntity()
                        .apply {
                            setOwner(userEntity)
                            setTopic(apiSub.topic)
                            setNumber(apiSub.estate_id)
                        }
            cachedSub.setState(apiSub.active)
            newSubList.add(cachedSub)
        }

        userSubs?.filter { !newSubList.map { s ->s.id }.contains(it.id)  }?.forEach {
            (it as SubscriptionEntity).setState(false)
        }

        userSubs?.forEach {s ->
            if (newSubList.firstOrNull { s.topic == it.topic && s.number == it.number} == null)
                newSubList.add(s as SubscriptionEntity)
        }

        return db.upsert(newSubList)
            .map {list -> list.map {
                fromEntity(
                it as SubscriptionEntity)
            }}
            .toObservable()
    }


    override fun subscribeTopicResult(
        user:UserEntity,
        subInnerId:Int,
        deviceToken: String,
        topic: String,
        topic_id: String?
    ): Observable<List<SubscriptionModel>> =
        api.subscribeTopic(
        userToken = user.authToken,
        deviceToken = deviceToken
        ,topicName = topic
        ,topicId = topic_id)
        .flatMap { saveSubscribtions(user,it.subscriptions?: arrayListOf()) }
        .doOnNext { db.refreshUser(user).blockingFirst() }

    override fun unSubscribeTopicResult(
        user:UserEntity,
        subInnerId:Int,
        deviceToken: String,
        topic: String,
        topic_id: String?
    ): Observable<List<SubscriptionModel>> =
        api.unSuubscribeTopic(
            userToken = user.authToken,
            deviceToken = deviceToken
            ,topicName = topic
            ,topicId = topic_id)
            .flatMap { saveSubscribtions(user,it.subscriptions?: arrayListOf()) }
            .doOnNext { db.refreshUser(user) }


    override fun subscribeTopic(
        user:UserEntity,
        subInnerId:Int,
        deviceToken: String,
        topic: String,
        topic_id: String?
    ): Completable = api.subscribeTopic(
        userToken = user.authToken,
        deviceToken = deviceToken
        ,topicName = topic
        ,topicId = topic_id)
        .flatMapCompletable {
            if (it.status == SUBSCRIPTION_RESULT_STATUS){
                saveSubscribeState(user,subInnerId,topic,topic_id,true)
                    .ignoreElements()
            }
            else error("Error subscription state!")
        }


    override fun unSubscribeTopic(
    user:UserEntity,
    subInnerId:Int,
    deviceToken: String,
    topic: String,
    topic_id: String?
    ): Completable =
        api.unSuubscribeTopic(
            userToken = user.authToken,
            deviceToken = deviceToken
            ,topicName = topic
            ,topicId = topic_id)
            .doOnNext { Timber.d("$it") }
            .flatMapCompletable  {
                if (it.status == SUBSCRIPTION_RESULT_STATUS){
                    saveSubscribeState(user,subInnerId,topic,topic_id,false)
                        .ignoreElements()
                }
                else error("Error subscription state!")
            }




}